==> stuff-0.0.7.tar.gz: lib/src/json_utils.dart <==
// Copyright (c) 2015, Anders Holmgren. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library stuff.json;

import 'package:option/option.dart';
import 'package:path/path.dart' as path;
import 'package:uri/uri.dart';

final _p = path.url;

abstract class Jsonable {
  toJson();
}

JsonBuilder get buildJson => new JsonBuilder();
JsonParser parseJson(Map j, {bool consumeMap: false}) =>
    new JsonParser(j, consumeMap);

class JsonBuilder {
  final bool _stringEmpties;
  JsonBuilder({bool stringEmpties: true}) : this._stringEmpties = stringEmpties;

  final Map json = {};

//  void addObject(String fieldName, o) {
//    if (o != null) {
//      json[fieldName] = o.toJson();
//    }
//  }

  void add(String fieldName, v, [transform(v)]) {
    if (v != null) {
      final transformed = _transformValue(v, transform);
      if (transformed != null) {
        json[fieldName] = transformed;
      }
    }
  }

  void addAll(Map map) {
    json.addAll(map);
  }

  _transformValue(value, [transform(v)]) {
    if (transform != null) {
      return transform(value);
    }
    if (value is Jsonable) {
      return value.toJson();
    }
    if (value is Map) {
      final result = {};
      value.forEach((k, v) {
        final transformedValue = _transformValue(v);
        final transformedKey = _transformValue(k);
        if (transformedValue != null && transformedKey != null) {
          result[transformedKey] = transformedValue;
        }
      });
      return result.isNotEmpty || !_stringEmpties ? result : null;
    }
    if (value is Option) {
      return value is Some ? _transformValue(value.get()) : null;
    }
    if (value is Iterable) {
      final list = value.map((v) => _transformValue(v, null)).toList();
      return list.isNotEmpty || !_stringEmpties ? list : null;
    }
    if (value is RegExp) {
      return value.pattern;
    }
    if (value is UriTemplate) {
      return value.template;
    }
    if (value is DateTime) {
      return value.toIso8601String();
    }
    if (value is bool || value is num) {
      return value;
    }
    return value.toString();
  }
}

typedef T Converter<T>(value);

Converter/*<T>*/ _converter/*<T>*/(Converter/*<T>*/ convert) =>
    convert ?? (v) => v as Object/*=T*/;

class JsonParser {
  final Map _json;
  final bool _consumeMap;

  JsonParser(Map json, bool consumeMap)
      : this._json = consumeMap ? new Map.from(json) : json,
        this._consumeMap = consumeMap;

  List/*<T>*/ list/*<T>*/(String fieldName, [Converter/*<T>*/ create]) {
    final List l = _getField(fieldName);
    return l != null ? l.map(_converter(create)).toList(growable: false) : [];
  }

  /*=T*/ single/*<T>*/(String fieldName, [/*=T*/ create(i)]) {
    final j = _getField(fieldName);
    return j != null ? _converter(create)(j) : null;
  }

  Map/*<K, V>*/ mapValues/*<K, V>*/(String fieldName,
      [Converter/*<V>*/ convertValue, Converter/*<K>*/ convertKey]) {
    final Map/*<K, V>*/ m = _getField/*<Map<K, V>>*/(fieldName);

    if (m == null) {
      return {};
    }

    Converter/*<K>*/ _convertKey = _converter(convertKey);
    Converter/*<V>*/ _convertValue = _converter(convertValue);

    Map/*<K, V>*/ result = new Map/*<K, V>*/();
    m.forEach((k, v) {
      result[_convertKey(k)] = _convertValue(v);
    });

    return result;
  }

  /*=T*/ _getField/*<T>*/(String fieldName) =>
      (_consumeMap ? _json.remove(fieldName) : _json[fieldName])
      as Object/*=T*/;

  Map get unconsumed {
    if (!_consumeMap) {
      throw new StateError('unconsumed called on non consuming parser');
    }

    return _json;
  }
}

==> stuff-0.1.0.tar.gz: lib/src/json_utils.dart <==
// Copyright (c) 2015, Anders Holmgren. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library stuff.json;

import 'package:uri/uri.dart';

abstract class Jsonable {
  toJson();
}

JsonBuilder get buildJson => new JsonBuilder();
JsonParser parseJson(Map j, {bool consumeMap: false}) =>
    new JsonParser(j, consumeMap);

class JsonBuilder {
  final bool _stringEmpties;
  JsonBuilder({bool stringEmpties: true}) : this._stringEmpties = stringEmpties;

  final Map json = {};

//  void addObject(String fieldName, o) {
//    if (o != null) {
//      json[fieldName] = o.toJson();
//    }
//  }

  void add(String fieldName, v, [transform(v)]) {
    if (v != null) {
      final transformed = _transformValue(v, transform);
      if (transformed != null) {
        json[fieldName] = transformed;
      }
    }
  }

  void addAll(Map map) {
    json.addAll(map);
  }

  _transformValue(value, [transform(v)]) {
    if (transform != null) {
      return transform(value);
    }
    if (value is Jsonable) {
      return value.toJson();
    }
    if (value is Map) {
      final result = {};
      value.forEach((k, v) {
        final transformedValue = _transformValue(v);
        final transformedKey = _transformValue(k);
        if (transformedValue != null && transformedKey != null) {
          result[transformedKey] = transformedValue;
        }
      });
      return result.isNotEmpty || !_stringEmpties ? result : null;
    }
    if (value is Iterable) {
      final list = value.map((v) => _transformValue(v, null)).toList();
      return list.isNotEmpty || !_stringEmpties ? list : null;
    }
    if (value is RegExp) {
      return value.pattern;
    }
    if (value is UriTemplate) {
      return value.template;
    }
    if (value is DateTime) {
      return value.toIso8601String();
    }
    if (value is bool || value is num) {
      return value;
    }
    return value.toString();
  }
}

typedef T Converter<T>(value);

Converter<T> _converter<T>(Converter<T> convert) =>
    convert ?? (v) => v as T;

class JsonParser {
  final Map _json;
  final bool _consumeMap;

  JsonParser(Map json, bool consumeMap)
      : this._json = consumeMap ? new Map.from(json) : json,
        this._consumeMap = consumeMap;

  List<T> list<T>(String fieldName, [Converter<T> create]) {
    final List l = _getField(fieldName);
    return l != null ? l.map(_converter(create)).toList(growable: false) : [];
  }

  T single<T>(String fieldName, [T create(i)]) {
    final j = _getField(fieldName);
    return j != null ? _converter(create)(j) : null;
  }

  Map<K, V> mapValues<K, V>(String fieldName,
      [Converter<V> convertValue, Converter<K> convertKey]) {
    final Map m = _getField(fieldName);

    if (m == null) {
      return {};
    }

    Converter<K> _convertKey = _converter(convertKey);
    Converter<V> _convertValue = _converter(convertValue);

    Map<K, V> result = new Map<K, V>();
    m.forEach((k, v) {
      result[_convertKey(k)] = _convertValue(v);
    });

    return result;
  }

  T _getField<T>(String fieldName) =>
      (_consumeMap ? _json.remove(fieldName) : _json[fieldName]);

  Map get unconsumed {
    if (!_consumeMap) {
      throw new StateError('unconsumed called on non consuming parser');
    }

    return _json;
  }
}
